@@ -60,9 +60,6 @@ html_chapters = function(
     self_contained = FALSE, lib_dir = lib_dir,
     template = template, pandoc_args = pandoc_args2(pandoc_args), ...
   )
-  if (pandoc2.0()) {
-    config$pandoc$to = 'html4'; config$pandoc$ext = '.html'
-  }
   split_by = match.arg(split_by)
   post = config$post_processor  # in case a post processor have been defined
   config$post_processor = function(metadata, input, output, clean, verbose) {
@@ -80,7 +77,7 @@ html_chapters = function(
 # add --wrap=preserve to pandoc args for pandoc 2.0:
 # https://github.com/rstudio/bookdown/issues/504
 pandoc_args2 = function(args) {
-  if (pandoc2.0()) c('--wrap', 'preserve', args) else args
+  if (pandoc2.0() && !length(grep('--wrap', args))) c('--wrap', 'preserve', args) else args
 }
 
 #' @rdname html_chapters
@@ -130,9 +127,6 @@ html_document2 = function(
   config = base_format(
     ..., number_sections = number_sections, pandoc_args = pandoc_args2(pandoc_args)
   )
-  if (pandoc2.0()) {
-    config$pandoc$to = 'html4'; config$pandoc$ext = '.html'
-  }
   post = config$post_processor  # in case a post processor have been defined
   config$post_processor = function(metadata, input, output, clean, verbose) {
     if (is.function(post)) output = post(metadata, input, output, clean, verbose)
@@ -140,6 +134,7 @@ html_document2 = function(
     x = restore_appendix_html(x, remove = FALSE)
     x = restore_part_html(x, remove = FALSE)
     x = resolve_refs_html(x, global = !number_sections)
+    x = clean_pandoc2_highlight_tags(x)
     write_utf8(x, output)
     output
   }
@@ -192,7 +187,8 @@ build_chapter = function(
     chapter,
     '<p style="text-align: center;">',
     button_link(link_prev, 'Previous'),
-    edit_link(rmd_cur),
+    source_link(rmd_cur, type = 'edit'),
+    source_link(rmd_cur, type = 'history'),
     button_link(link_next, 'Next'),
     '</p>',
     '</div>',
@@ -212,6 +208,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   if (!(split_level %in% 0:2)) stop('split_level must be 0, 1, or 2')
 
   x = read_utf8(output)
+  x = clean_meta_tags(x)
 
   i1 = find_token(x, '<!--bookdown:title:start-->')
   i2 = find_token(x, '<!--bookdown:title:end-->')
@@ -267,6 +264,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   x = add_section_ids(x)
   x = restore_part_html(x)
   x = restore_appendix_html(x)
+  x = clean_pandoc2_highlight_tags(x)
 
   # no (or not enough) tokens found in the template
   if (any(c(i1, i2, i3, i4, i5, i6) == 0)) {
@@ -344,7 +342,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
       if (is.null(nm)) stop('The heading ', x2, ' must have an id')
       gsub('[^[:alnum:]]+', '-', nm)
     })
-    if (anyDuplicated(nms)) stop(
+    if (anyDuplicated(nms)) (if (isTRUE(opts$get('preview'))) warning else stop)(
       'Automatically generated filenames contain duplicated ones: ',
       paste(nms[duplicated(nms)], collapse = ', ')
     )
@@ -379,7 +377,7 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
     html = relocate_footnotes(html, fnts, a_targets)
     html = restore_links(html, html_body, idx, nms)
     html = build(
-      html_head, html_toc, html,
+      prepend_chapter_title(html_head, html), html_toc, html,
       if (i > 1) nms[i - 1],
       if (i < n) nms[i + 1],
       if (length(nms_chaps)) nms_chaps[i],
@@ -396,6 +394,20 @@ split_chapters = function(output, build = build_chapter, number_sections, split_
   nms[j]
 }
 
+# clean HTML tags inside <meta>, which can be introduced by certain YAML
+# metadata, such as an improper description that contains Markdown syntax, e.g.,
+# <meta name="description" content="A <i>description</i>.">
+clean_meta_tags = function(x) {
+  r = '^(\\s*<meta )(.+<.+>.+)(/?>\\s*)$'
+  if (length(i <- grep(r, x)) == 0) return(x)
+  x1 = sub(r, '\\1', x[i])
+  x2 = sub(r, '\\2', x[i])
+  x3 = sub(r, '\\3', x[i])
+  x2 = gsub('<[^>]+>', '', x2)
+  x[i] = paste0(x1, x2, x3)
+  x
+}
+
 # move files to output dir if specified
 move_to_output_dir = function(files) {
   files2 = output_path(files)
@@ -419,24 +431,40 @@ button_link = function(target, text) {
   )
 }
 
-edit_link = function(target) {
+source_link = function(target, type) {
   if (length(target) == 0) return()
-  setting = edit_setting()
+  setting = source_link_setting(type = type)
   if (is.null(setting)) return()
   button_link(sprintf(setting$link, target), setting$text)
 }
 
-edit_setting = function(config) {
-  config_default = load_config()[['edit']]
+source_link_setting = function(config, type) {
+  config_default = load_config()[[type]]
   if (missing(config) || is.null(config)) config = config_default
   if (is.null(config)) return()
   if (is.character(config)) config = list(link = config, text = NULL)
   if (!is.character(link <- config[['link']])) return()
-  if (!grepl('%s', link)) stop('The edit link must contain %s')
-  if (!is.character(text <- config[['text']])) text = ui_language('edit')
+  if (!grepl('%s', link)) stop('The ', type, ' link must contain %s')
+  if (!is.character(text <- config[['text']])) text = ui_language(type)
   list(link = link, text = text)
 }
 
+#' Resolve figure/table/section references in HTML
+#'
+#' Post-process the HTML content to resolve references of figures, tables, and
+#' sections, etc. The references are written in the form \code{\@ref(key)} in
+#' the R Markdown source document.
+#' @param content A character vector of the HTML content.
+#' @param global Whether to number the elements incrementally throughout the
+#'   whole document, or number them by the first-level headers. For an R
+#'   Markdown output format, \code{global = !number_sections} is likely to be a
+#'   reasonable default (i.e., when the sections are numbered, you number
+#'   figures/tables by sections; otherwise just number them incrementally).
+#' @return A post-processed character vector of the HTML character.
+#' @export
+#' @keywords internal
+#' @examples library(bookdown)
+#' resolve_refs_html(c('<caption>(#tab:foo) A nice table.</caption>', '<p>See Table @ref(tab:foo).</p>'), TRUE)
 resolve_refs_html = function(content, global = FALSE) {
   content = resolve_ref_links_html(content)
 
@@ -465,14 +493,6 @@ is_img_line = function(x) grepl('^<img src=".* alt="', x)
 ref_to_number = function(ref, ref_table, backslash) {
   if (length(ref) == 0) return(ref)
   lab = gsub(if (backslash) '^\\\\@ref\\(|\\)$' else '^@ref\\(|\\)$', '', ref)
-  # TODO: deprecate the prefix ex: for Examples (use exm: instead)
-  if (length(i <- grep('^ex:.+', lab))) {
-    warning(
-      'Please change the prefix ex: to exm: in label(s) ', knitr::combine_words(lab[i]),
-      call. = FALSE
-    )
-    lab[i] = sub('^ex:', 'exm:', lab[i])
-  }
   ref = prefix_section_labels(lab)
   num = ref_table[ref]
   i = is.na(num)
@@ -503,12 +523,12 @@ reg_chap = '^(<h1><span class="header-section-number">)([A-Z0-9]+)(</span>.+</h1
 label_names = list(fig = 'Figure ', tab = 'Table ', eq = 'Equation ')
 # prefixes for theorem environments
 theorem_abbr = c(
-  theorem = 'thm', lemma = 'lem', definition = 'def', corollary = 'cor',
-  proposition = 'prp', example = 'exm', exercise = 'exr'
+  theorem = 'thm', lemma = 'lem', corollary = 'cor', proposition = 'prp', conjecture = 'cnj',
+  definition = 'def', example = 'exm', exercise = 'exr'
 )
 # numbered math environments
 label_names_math = setNames(list(
-  'Theorem ', 'Lemma ', 'Definition ', 'Corollary ', 'Proposition ', 'Example ', 'Exercise '
+  'Theorem ', 'Lemma ', 'Corollary ', 'Proposition ', 'Conjecture ', 'Definition ', 'Example ', 'Exercise '
 ), theorem_abbr)
 # unnumbered math environments
 label_names_math2 = list(proof = 'Proof. ', remark = 'Remark. ', solution = 'Solution. ')
@@ -755,7 +775,7 @@ restore_part_html = function(x, remove = TRUE) {
     k = grepl('^<h1>\\(PART\\*\\) .+</h1>$', x[i])  # unnumbered parts
     i1 = i[k]; i2 = i[!k]
     x[i1] = gsub('<h1>\\(PART\\*\\)', '<h1>', x[i1])
-    x[i2] = mapply(
+    if (length(i2)) x[i2] = mapply(
       gsub, '<h1>\\(PART\\)', x = x[i2],
       sprintf('<h1><span class="header-section-number">%s</span>', as.roman(seq_along(i2)))
     )
@@ -895,6 +915,7 @@ move_files_html = function(output, lib_dir) {
     if (length(z) == 0) z else gsub(r, '\\2', z)
   }))
   f = c(f, parse_cover_image(x))
+  f = vapply(f, utils::URLdecode, FUN.VALUE = character(1))
   f = local_resources(unique(f[file.exists(f)]))
   # detect resources in CSS
   css = lapply(grep('[.]css$', f, ignore.case = TRUE, value = TRUE), function(z) {
@@ -974,3 +995,32 @@ restore_math_labels = function(x) {
   x[i] = gsub('\\(\\\\(#eq:[-/[:alnum:]]+)\\)', '(\\1)', x[i])
   x
 }
+
+# remove the <div> tags around <pre>, and clean up <a> on all lines
+clean_pandoc2_highlight_tags = function(x) {
+  if (!pandoc2.0()) return(x)
+  x = gsub('(</a></code></pre>)</div>', '\\1', x)
+  x = gsub('<div class="sourceCode"[^>]+>(<pre)', '\\1', x)
+  x = gsub('<a class="sourceLine"[^>]+>(.*)</a>', '\\1', x)
+  x
+}
+
+# extract a chapter title from the body, and prepend it to the page <title>
+prepend_chapter_title = function(head, body) {
+  r1 = '(.*?<title>)(.+?)(</title>.*)'
+  if (length(i <- grep(r1, head)) == 0) return(head)
+  body = paste(body, collapse = ' ')
+  r2 = '.*?<h[0-6][^>]*>(.+?)</h[0-6]>.*'
+  if (!grepl(r2, body)) return(head)
+  title = strip_html(gsub(r2, '\\1', body))
+  x1 = gsub(r1, '\\1', head[i])
+  x2 = gsub(r1, '\\2', head[i])
+  x3 = gsub(r1, '\\3', head[i])
+  if (title == x2) return(head)
+  head[i] = paste0(x1, title, ' | ', x2, x3)
+  # update possible OpenGraph tags <meta property="og:title" content="...">
+  gsub(
+    paste0(' content="', x2, '"'), paste0(' content="', title, ' | ', x2),
+    head, fixed = TRUE
+  )
+}
